/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/
#ifndef RAYTRACER_H
#define RAYTRACER_H

#include "GlutDemoApplication.h"

class btDefaultCollisionConfiguration;
class btCollisionDispatcher;
class btAxisSweep3;
class btCollisionWorld;
class btDbvtBroadphase;


// Just a try... needs further testing and fixing...
class btCollisionShapeGarbageCollector : public btAlignedObjectArray<btCollisionShape*> {
	public:
	typedef btAlignedObjectArray<btCollisionShape*> Base;
	
	btCollisionShapeGarbageCollector() : Base() {}
	bool push_back(btCollisionShape* s,bool dontInsertDoublesAndNulls=false)	{
		if (!dontInsertDoublesAndNulls) {Base::push_back(s);return true;}
		else if (s && Find(s)==-1)  {Base::push_back(s);return true;}
		return false; 
	}
	
	~btCollisionShapeGarbageCollector()	{
		clear();
	}
	
	void clear() {
		deleteAll();		
	}
	
	protected:
	
	int Find(btCollisionShape* s)	{
		const int a = Base::findLinearSearch(s);
		return (a == size()) ? -1 : a;
	}
	
	void deleteAll();
	
	void cleanList();
	
	private:
	btCollisionShapeGarbageCollector(const btCollisionShapeGarbageCollector& c) {}
	btCollisionShapeGarbageCollector(const Base& c) {}
	const btCollisionShapeGarbageCollector* operator=(const btCollisionShapeGarbageCollector& c)	{}
	const btCollisionShapeGarbageCollector* operator=(const Base& c)	{}
	void resize(int i) {}
	void pop_back() {}
	void remove(const btCollisionShape* s) {}
	void removeDoublesAndNulls();
	void flattenHierarchy(bool dontInsertDoublesAndNulls);
	void addChildShapesOf(btCollisionShape* shape,bool dontInsertDoublesAndNulls,bool addThePassedShapeIfLeafShape=false);
};


///Raytracer shows the inner working of the ray casting, using ray tracing rendering into a texture.
class Raytracer : public GlutDemoApplication
{

	btDefaultCollisionConfiguration*	m_collisionConfiguration;
	btCollisionDispatcher*	m_dispatcher;
	btDbvtBroadphase*	m_overlappingPairCache;
	btCollisionWorld*	m_collisionWorld;
	bool	m_initialized;
	btScalar m_orthoCameraFov;
	btScalar m_orthoCameraFocusDistance;	// Initial value : PgUp and PgDw change it
	btScalar m_minCameraDistance;			// If too low, raytrace images appear clipped
	int m_curFrameTimer;
	
	public:
	btCollisionShapeGarbageCollector m_collisionShapeGarbageCollector;
	
	Raytracer() : GlutDemoApplication(),m_orthoCameraFov(SIMD_HALF_PI),m_orthoCameraFocusDistance(20),m_minCameraDistance(30),m_curFrameTimer(1)	{}

	void	initPhysics();

	virtual ~Raytracer();

	virtual void clientMoveAndDisplay()	{}

	virtual void displayCallback();

	//// casts rays and updates the texture
	void updateRaytraceTexture(int textureIndex = 1);

	static DemoApplication* Create()
	{
		Raytracer* demo = new Raytracer();
		demo->myinit();
		demo->initPhysics();
		return demo;
	}	
	
	protected:
	
	virtual void initWorld();
	
	void createTexture(int textureIndex = 1);
	void updateTexture(int textureIndex = 1);

	void specialKeyboard(int key, int x, int y);
};


#endif //RAYTRACER_H


